function math.clamp(a,min,max)
	return math.max(math.min(a,max),min)
end

GUIS = {
	Objects = {},
	
	newSlider = function(name,px,py,sx,sy,minv,maxv,curv)
		Slider = {
			Name         = name,
			Position     = {x=px,y=py},
			Size         = {x=sx,y=sy},
			MinValue     = minv,
			MaxValue     = maxv,
			Value        = math.clamp(curv,minv,maxv),
			
			MouseLeftInteracting  = false,
			MouseRightInteracting = false,
					
			setValueFromMousePosition = function(self,mx,my)

			end,
			
			getSliderSize = function(self)
			
			end,
			
			draw = function(self)
				love.graphics.setColor(0,0,0,255)
				love.graphics.rectangle("fill",self.Position.x,self.Position.y,self.Size.x,self.Size.y)
				
				love.graphics.setColor(100,100,255,255)
				love.graphics.rectangle("fill",self.Position.x,self.Position.y,self:getSliderSize(),self.Size.y)
				
				love.graphics.setColor(255,255,255,255)
				love.graphics.rectangle("line",self.Position.x,self.Position.y,self.Size.x,self.Size.y)

				love.graphics.setColor(255,255,255,255)
				love.graphics.printf(tostring(self.Value),self.Position.x,self.Position.y+self.Size.y/2-7.5,self.Size.x,"center")
			end,
			
			mouseLeftClickEvents = {},
			
			mousepressed = function(self,mx,my,button)
				if (mx>self.Position.x and my>self.Position.y and mx<self.Position.x+self.Size.x and my<self.Position.y+self.Size.y) then
					print("GUIS : "..self.Name.." : Got clicked with button "..button)
					if (button==1) then
						self.MouseLeftInteracting = true
						self:setValueFromMousePosition(mx,my)
						
						for i,e in pairs(self.mouseLeftClickEvents) do e() end
						
						return true
					elseif (button==2) then
						self.MouseRightInteracting = true
						return true
					end
				end
				return false
			end,
			
			
			mousemoved = function(self,mx,my,dx,dy)
				if (self.MouseLeftInteracting) then
					print("GUIS : "..self.Name.." : Currently Mouse Left Click Interacting ")
					self:setValueFromMousePosition(mx,my)
				elseif (self.MouseRightInteracting) then
					print("GUIS : "..self.Name.." : Currently Mouse Right Click Interacting ")
					p = self.Position.x+self:getSliderSize()+dx*.1
					self:setValueFromMousePosition(p,my)
				end
			end,
			
			
			mousereleased = function(self,mx,my,button)
				if(button==1)then
					self.MouseLeftInteracting = false
				elseif(button==2)then
					self.MouseRightInteracting = false
				end
			end,
			
			update = function(self)

			end,
		}
		table.insert(GUIS.Objects,Slider)
		return Slider
	end,
	
	newLineairSlider = function(name,px,py,sx,sy,minv,maxv,curv)
		n = GUIS.newSlider(name,px,py,sx,sy,minv,maxv,curv)
		
		n.setValueFromMousePosition = function(self,mx,my)
			t = (mx-self.Position.x)/self.Size.x
			self.Value = self.MinValue + t*(self.MaxValue-self.MinValue)
			self.Value = math.clamp(self.Value, self.MinValue, self.MaxValue)
		end
			
		n.getSliderSize = function(self)
			return self.Size.x*(self.Value-self.MinValue)/(self.MaxValue-self.MinValue)
		end
		
		return n
	end,
	
	newLogarithmicSlider = function(name,px,py,sx,sy,minv,maxv,curv)	
		n = GUIS.newSlider(name,px,py,sx,sy,minv,maxv,curv)
			
		n.setValueFromMousePosition = function(self,mx,my)
			t = (mx-self.Position.x)/self.Size.x
			self.Value = self.MinValue*math.exp(t*math.log(self.MaxValue/self.MinValue))
			self.Value = math.clamp(self.Value, self.MinValue, self.MaxValue)
		end
		
		n.getSliderSize = function(self)
			return self.Size.x*math.log(self.Value/self.MinValue)/math.log(self.MaxValue/self.MinValue)
		end
			
		return n
	end,
	
	newIntegerSlider = function(name,px,py,sx,sy,minv,maxv,curv)	
		n = GUIS.newSlider(name,px,py,sx,sy,math.floor(minv),math.ceil(maxv),math.floor(curv+.5))
		
		n.setValueFromMousePosition = function(self,mx,my)
			t = (mx-self.Position.x)/self.Size.x
			self.Value = math.floor(self.MinValue + t*(self.MaxValue-self.MinValue)+.5)
			self.Value = math.clamp(self.Value, self.MinValue, self.MaxValue)
		end
			
		n.getSliderSize = function(self)
			return self.Size.x*(self.Value-self.MinValue)/(self.MaxValue-self.MinValue)
		end
		
		return n
	end,
	
	mousepressed = function(self,mx,my,button)
		pressedsomething = false
		for i,o in pairs(self.Objects) do
			pressedsomething = pressedsomething or o:mousepressed(mx,my,button)
		end
		return pressedsomething
	end,
	
	mousemoved = function(self,mx,my,dx,dy)
		for i,o in pairs(self.Objects) do
			o:mousemoved(mx,my,dx,dy)
		end
	end,
	
	mousereleased = function(self,mx,my,button)
		for i,o in pairs(self.Objects) do
			o:mousereleased(mx,my,button)
		end
	end,
	
	update = function(self)
		for i,o in pairs(self.Objects) do
			o:update()
		end	
	end,
	
	draw = function(self)
		for i,o in pairs(self.Objects) do
			o:draw()
		end
	end
	
}
