require("vector")
require("GUIS")

function math.clamp(a,min,max)
	return math.min(math.max(a,min),max)
end

function love.load()
	Width,Height = love.graphics.getDimensions()
	
	Fractal = {
		OffsetX    = GUIS.newLineairSlider("OffsetX"   ,120,135,280,15,0 ,1.       ,.5),
		OffsetY    = GUIS.newLineairSlider("OffsetY"   ,120,150,280,15,0 ,1.       ,.5),
		OffsetZ    = GUIS.newLineairSlider("OffsetZ"   ,120,165,280,15,0 ,1.       ,.5),
		
		RotationX  = GUIS.newLineairSlider("RotationX" ,120,180,280,15,-math.pi, math.pi,0),
		RotationY  = GUIS.newLineairSlider("RotationY" ,120,195,280,15,-math.pi, math.pi,0),
		RotationZ  = GUIS.newLineairSlider("RotationZ" ,120,210,280,15,-math.pi, math.pi,0),
		
		Iterations = GUIS.newIntegerSlider("Iterations",120,225,280,15,0 ,10       ,5),
	}

	Camera = {
		Position    = Vector.new(0,0,3),
		Velocity    = Vector.new(0,0,0),
		Forward     = Vector.new(0,0,-1),
		Right       = Vector.new(-1,0,0),
		Upward      = Vector.new(0,1,0),
		Aperture    = GUIS.newLogarithmicSlider("Aperture",120,90 ,280,15,1e-12,1 ,0.001),
		Focal       = GUIS.newLogarithmicSlider("Focal"   ,120,105,280,15,1e-4 ,10 ,2    )
	}
	
	Sun = {
		Longitude = GUIS.newLineairSlider    ("SunLongitude",120,255,280,15,-180,180,40),
		Latitude  = GUIS.newLineairSlider    ("SunLatitude" ,120,270,280,15,0,90,60),
		Size      = GUIS.newLogarithmicSlider("SunSize"     ,120,285,280,15,1e-6,.1,0.001),
	}
	
	Render = {
		ResolutionX = GUIS.newIntegerSlider  ("ResolutionX",120,345,280,15,0,2048,1920),
		ResolutionY = GUIS.newIntegerSlider  ("ResolutionY",120,360,280,15,0,2048,1080)
	}
	
	MovementSpeed     = 1.
	
	RenderCanvas      = love.graphics.newCanvas(Render.ResolutionX.Value,Render.ResolutionY.Value,rgba32f)
	Frames            = 0;
	
	RenderShader      = love.graphics.newShader("mengersponge.frag")
	PostShader        = love.graphics.newShader("posteffect.frag")
	
	startTime         = love.timer.getTime()
	
	Rendering         = false;
	ShowInfo          = true;
	CameraDragging    = false
end

function ResetRender()
	startTime    = love.timer.getTime()
	RenderCanvas = love.graphics.newCanvas(Render.ResolutionX.Value,Render.ResolutionY.Value,rgba32f)
	Frames       = 0;
end

function UpdateShader()
	if (Rendering) then
		RenderShader:send('iResolution',{Render.ResolutionX.Value,Render.ResolutionY.Value})
	else
		RenderShader:send('iResolution',{Width,Height})
	end
    RenderShader:send('iGlobalTime',love.timer.getTime()-startTime)
	
	RenderShader:send('CameraPosition'   ,Vector.keyless(Camera.Position))
	RenderShader:send('CameraForward'    ,Vector.keyless(Camera.Forward))
	RenderShader:send('CameraUpward'     ,Vector.keyless(Camera.Upward))
	RenderShader:send('CameraRight'      ,Vector.keyless(Camera.Right))
	RenderShader:send('CameraAperture'   ,Camera.Aperture.Value)
	RenderShader:send('CameraFocal'      ,Camera.Focal.Value)

	RenderShader:send('FractalOffset'    ,{Fractal.OffsetX.Value  ,Fractal.OffsetY.Value  ,Fractal.OffsetZ.Value  })
	RenderShader:send('FractalRotation'  ,{Fractal.RotationX.Value,Fractal.RotationY.Value,Fractal.RotationZ.Value})
	RenderShader:send('FractalIterations',Fractal.Iterations.Value)
	
	RenderShader:send('SunVector'        ,Vector.keyless(Vector.spherical(Sun.Longitude.Value/180*math.pi,Sun.Latitude.Value/180*math.pi)))
	RenderShader:send('SunSize'          ,Sun.Size.Value)
	
	RenderShader:send('Rendering'        ,Rendering)
end

function love.resize(w, h)
	print("Window resized to "..w..","..h)
	Width = w
	Height = h
end

function love.mousepressed(x,y,button)
	if (not Rendering) then
		if (ShowInfo) then
			GUIInteract = GUIS:mousepressed(x,y,button)
			if (not GUIInteract and button == 1) then
				CameraDragging = true
			end
		else
			if (button==1) then
				CameraDragging = true
			end
		end
	end
end

function love.mousemoved(x,y,dx,dy)
	GUIS:mousemoved(x,y,dx,dy)
end

function love.mousereleased(x,y,button)
	GUIS:mousereleased(x,y,button)
	if (button==1) then
		CameraDragging = false
	end
end

function love.keypressed(key, scancode)
	if scancode == "b" then
		if (Rendering) then
			
			--Still need to apply post processing to the rendercanvas
			ScreenShotCanvas = love.graphics.newCanvas(Render.ResolutionX.Value,Render.ResolutionY.Value,rgba32f)
			love.graphics.setCanvas(ScreenShotCanvas)
				love.graphics.setShader(PostShader)
					love.graphics.draw(RenderCanvas)
				love.graphics.setShader()
			love.graphics.setCanvas()
			
			--take the screenshot
			ScreenShotCanvas:newImageData():encode('png', os.time() .. '.png');
		else
			love.graphics.newScreenshot( ):encode('png', os.time() .. '.png');
		end
		
		print("Screenshot taking!")
	elseif scancode == "i" then
		ShowInfo = not ShowInfo
	elseif scancode == "f" then
		Rendering = not Rendering
		ResetRender()
	end
end

function love.wheelmoved(x, y)
    if y > 0 then
        MovementSpeed = MovementSpeed*2;
    elseif y < 0 then
        MovementSpeed = MovementSpeed/2;
    end
	--limiting
	MovementSpeed = math.clamp(MovementSpeed,0.0001,2)
end
 

function love.update(dt)
	if(not Rendering)then
		--Direction
		if (CameraDragging) then
			local x, y = love.mouse.getPosition()

			x, y = ( x * 2 - Width ) / Height, ( y * 2 - Height ) / Height
				
			Camera.Forward = Vector.rotate(Camera.Forward, x * dt, Camera.Upward )
			Camera.Forward = Vector.rotate(Camera.Forward, y * dt, Camera.Right  )

			Camera.Upward = Vector.normalize(Vector.cross(Camera.Forward, Camera.Right ))
			Camera.Right  = Vector.normalize(Vector.cross(Camera.Upward, Camera.Forward ))
		end
		
		if love.keyboard.isScancodeDown("w") then
			Camera.Velocity = Camera.Velocity + Camera.Forward*(dt*MovementSpeed)
		end
		if love.keyboard.isScancodeDown("s") then
			Camera.Velocity = Camera.Velocity - Camera.Forward*(dt*MovementSpeed)
		end
		if love.keyboard.isScancodeDown("d") then
			Camera.Velocity = Camera.Velocity + Camera.Right*(dt*MovementSpeed)
		end
		if love.keyboard.isScancodeDown("a") then
			Camera.Velocity = Camera.Velocity - Camera.Right*(dt*MovementSpeed)
		end
		if love.keyboard.isScancodeDown("space") then
			Camera.Velocity = Camera.Velocity + Camera.Upward*(dt*MovementSpeed)
		end
		if love.keyboard.isScancodeDown("lshift") then
			Camera.Velocity = Camera.Velocity - Camera.Upward*(dt*MovementSpeed)
		end
		
		--Limit velocity
		local AbsoluteVelocity = Vector.length(Camera.Velocity)
		if(AbsoluteVelocity>0)then
			Camera.Velocity = Camera.Velocity*(math.min(MovementSpeed,AbsoluteVelocity)/AbsoluteVelocity)
		end
	
		--Drag
		Camera.Velocity = Camera.Velocity*(0.5^dt)
	
		--Integrate Camera.Velocity
		Camera.Position = Camera.Position + Camera.Velocity*dt
	end
	
	--normalize Offset (multiples of Offset only result in bigger shape, so different results lay on a sphere)
	local l = math.sqrt(Fractal.OffsetX.Value^2+Fractal.OffsetY.Value^2+Fractal.OffsetZ.Value^2)
	Fractal.OffsetX.Value = Fractal.OffsetX.Value/l
	Fractal.OffsetY.Value = Fractal.OffsetY.Value/l
	Fractal.OffsetZ.Value = Fractal.OffsetZ.Value/l
	
	UpdateShader()
end

function love.draw()

	if (Rendering) then
	
		local crenderw,crendery = RenderCanvas:getDimensions()
		if(not (Render.ResolutionX.Value == crenderw and Render.ResolutionY.Value == crendery)) then
			ResetRender()
		end

		--Create a new frame canvas
		local newRenderFrame = love.graphics.newCanvas(Render.ResolutionX.Value,Render.ResolutionY.Value,rgab32f)
		love.graphics.setCanvas(newRenderFrame)
			--render frame
			love.graphics.setShader(RenderShader)
				love.graphics.rectangle("fill",0,0,Render.ResolutionX.Value,Render.ResolutionY.Value)
			love.graphics.setShader()
		love.graphics.setCanvas()
		
		--Add it to the Rendering Canvas
		love.graphics.setCanvas(RenderCanvas)
			love.graphics.setColor(255,255,255,255/(Frames+1))
			love.graphics.draw(newRenderFrame)
		love.graphics.setCanvas()
		
		--post process filter
		love.graphics.setShader(PostShader)
			--draw rendercanvas to window
			local scale = math.min(Width/Render.ResolutionX.Value,Height/Render.ResolutionY.Value)
			love.graphics.draw(RenderCanvas,Width/2.,Height/2.,0,scale,scale,Render.ResolutionX.Value/2,Render.ResolutionY.Value/2)--first offset is applied, then scale!
		love.graphics.setShader()
		
	
		print("Rendered Frame "..Frames)
		Frames = Frames + 1
		
	else
	
		--create new frame
		local newRenderFrame = love.graphics.newCanvas(Width,Height)
		love.graphics.setCanvas(newRenderFrame)
			--render frame
			love.graphics.setShader(RenderShader)
				love.graphics.rectangle("fill",0,0,Width,Height)
			love.graphics.setShader()
		love.graphics.setCanvas()
		
		--post process filter
		love.graphics.setShader(PostShader)
			--draw frame to window
			love.graphics.draw(newRenderFrame)
		love.graphics.setShader()
		
	end
	
	--extra info
	if (ShowInfo) then
		love.graphics.setColor(0,0,0)
		love.graphics.rectangle("fill",0,0,400,375)
		love.graphics.setColor(255,255,255)
		
		love.graphics.print("press 'b' to make a screenshot",0,0)
		love.graphics.print("press 'i' to remove this info",0,15)
		love.graphics.print("press 'f' to switch to ULTIMATE Rendering MODE",0,30)
		
		love.graphics.print("Camera Position (WASD+Shift+Space) : "..Vector.tostring(Camera.Position,100),0,60)
		love.graphics.print("Camera Forward (Mouse Drag) : "..Vector.tostring(Camera.Forward,100),0,75)
		love.graphics.print("Camera Aperture : ",0,90)
		love.graphics.print("Camera Focal : ",0,105)
		
		love.graphics.print("Fractal Offset : ",0,135)
		love.graphics.print("Fractal Rotation : ",0,180)
		love.graphics.print("Fractal Iterations : ",0,225)
		
		love.graphics.print("Sun Longitude : ",0,255)
		love.graphics.print("Sun Latitude : ",0,270)
		love.graphics.print("Sun Size : ",0,285)
		
		love.graphics.print("Movement Speed (mousewheel) : "..MovementSpeed,0,315)
		
		love.graphics.print("Render Resolution : ",0,345)
		
		GUIS:draw()
	end
	
end