Vector = {
	new = function(x,y,z)
		v = {x=x,y=y,z=z}
		setmetatable(v,Vector.metatable)
		return v
	end,
	
	length = function(v)
		return math.sqrt(v.x^2+v.y^2+v.z^2)
	end,
	
	normalize = function(v)
		return v/Vector.length(v)
	end,
	
	cross = function(v1,v2)
		return Vector.new(v1.y*v2.z-v1.z*v2.y,
				v1.z*v2.x-v1.x*v2.z,
				v1.x*v2.y-v1.y*v2.x)
	end,
	
	spherical = function(p,t)
		return Vector.new(
			math.cos(t)*math.sin(p),
			math.sin(t),
			math.cos(t)*math.cos(p)
		)
	end,
	
	max = function(v1,v2)
		return Vector.new(
			math.max(v1.x,v2.x),
			math.max(v1.y,v2.y),
			math.max(v1.z,v2.z)
		)
	end,
	
	min = function(v1,v2)
		return Vector.new(
			math.min(v1.x,v2.x),
			math.min(v1.y,v2.y),
			math.min(v1.z,v2.z)
		)
	end,
	
	clamp = function(v1,v2,v3)
		return Vector.max(Vector.min(v1,v3),v2)
	end,
	
	rotate = function( v1, angle, v2 )--rotates v1 around v2 with angle radians
		local c = math.cos( angle )
		local s = math.sin( angle )

		return Vector.new(
				  v1.x * ( c + v2.x * v2.x * ( 1 - c )            ) +
				  v1.y * (     v2.x * v2.y * ( 1 - c ) - v2.z * s ) +
				  v1.z * (     v2.x * v2.z * ( 1 - c ) + v2.y * s ),

				  v1.x * (     v2.y * v2.x * ( 1 - c ) + v2.z * s ) +
				  v1.y * ( c + v2.y * v2.y * ( 1 - c )            ) +
				  v1.z * (     v2.y * v2.z * ( 1 - c ) - v2.x * s ),

				  v1.x * (     v2.z * v2.x * ( 1 - c ) - v2.y * s ) +
				  v1.y * (     v2.z * v2.y * ( 1 - c ) + v2.x * s ) +
				  v1.z * ( c + v2.z * v2.z * ( 1 - c )            )
		)
	end,
	
	tostring = function(v,d)
		return math.floor(v.x*d)/d..","..math.floor(v.y*d)/d..","..math.floor(v.z*d)/d
	end,
	
	keyless = function(v)
		return {v.x,v.y,v.z}
	end
}
	
Vector.metatable = {
	__add = function(v1,v2) return Vector.new(v1.x+v2.x, v1.y+v2.y, v1.z+v2.z) end,
	__sub = function(v1,v2) return Vector.new(v1.x-v2.x, v1.y-v2.y, v1.z-v2.z) end,
	__mod = function(v1,v2) return Vector.new(v1.x%v2.x, v1.y%v2.y, v1.z%v2.z) end,
	__pow = function(v1,v2) return Vector.new(v1.x^v2.x, v1.y^v2.y, v1.z^v2.z) end,
	__mul = function(v1,c)  return Vector.new(v1.x*c, v1.y*c, v1.z*c) end,
	__div = function(v1,c)  return Vector.new(v1.x/c, v1.y/c, v1.z/c) end
}




